/** Represents an array instance within a TableOfVars instance */
function TableOfVarsArray(
    parentTable,    // TableOfVarsAbstract instance containing this instance
    baseType,       // Array items' variable (base) type
    dimensions,     // Dimension of array
    length,         // Count of array items
    pointer         // Optional: Preferred memory address
) {
    this.baseType = baseType;
    this.dimensions = dimensions;
    this.calcSize(length, parentTable.varTypes);
    TableOfVarsArray.parent.call(this, parentTable, pointer);

    const doc = this.getDocument();
    const tr = this.createTr(
        this.createCellContentName(doc, 'length'),
        this.createCellContentType(doc, 'int'),
        this.createCellContentNumber(doc, length, true)
    );
    this.nodeData[0].parentNode.insertBefore(tr, this.nodeData[0].nextSibling);
    this.nodeData.push(tr);

    const atts = [];
    while (atts.length < length)
        atts.push( { key: atts.length, type: baseType, dimensions: Math.max(0, dimensions - 1) } );
    this.addVars(atts);
}
TableOfVarsArray.parent = TableOfVarsInstancelike;
TableOfVarsArray.prototype = Object.create(TableOfVarsArray.parent.prototype);
TableOfVarsArray.prototype.constructor = TableOfVarsArray;

/**
* Create DOM node representing number
* @return Node      DOM node beeing type cell's content
*/
TableOfVarsArray.prototype.createCellContentNumber = function(
    doc,        // Document instance
    number,     // Number to display
    noBrackets  // Do not bracket number with square brackets?
) {
    const nodeCode = doc.createElement('code');
    if (!noBrackets)
        ScoTools.createHtml(false, nodeCode, null, '[');
    ScoTools.createHtml('span', nodeCode, { 'class': 'hljs-number' }, number);
    if (!noBrackets)
        ScoTools.createHtml(false, nodeCode, null, ']');
    return nodeCode;
}

/**
* Set this.size by calculating size of attributes
* @return undefined
*/
TableOfVarsArray.prototype.calcSize = function(
    length,         // Count of array items
    varTypes        // Information on base variable types (see TableOfVars.varTypes)
) {
    this.size = VarTypeInt.countBytes;
    if (length <= 0)
        return;
    const varClass = (this.dimensions <= 1) ? varTypes.reduce((prev, curr) => (prev || ((curr.typeName == this.baseType) ? curr : null)), null) : null;
    this.size += length * (varClass ? varClass.countBytes : VarTypeReference.countBytes);
}

/** Description of this type in error messages */
TableOfVarsArray.prototype.descType = 'Array';

/**
* Convert contained data to JSON
* @return Object    JSON representation of contained data
*/
TableOfVarsArray.prototype.toJSON = function() {
    const json = {
        instance: 'array',
        baseType: this.baseType,
        dimensions: this.dimensions,
        values: this.vars.map(variable => variable.toJSON().value),
        pointer: this.pointer
    };
    const values = this.vars.map(variable => variable.toJSON());
    if (values.length && (values[0].ref !== undefined))
        json.refs = values.map(obj => obj.ref);
    else
        json.values = values.map(obj => obj.value);
    return json;
}

/**
* Replaces current variables by a new set of variables
* @return Array     Array of booleans representing success of adding corresponding variable
*/
TableOfVarsArray.prototype.load = function(
    values,         // Array of values
    instanceMapping // Optional: Object mapping saved memory pointer to actual pointers
) {
    if (!(values instanceof Array))
        return [ false ];
    return values.map((value, index) => {
        if (index >= this.vars.length)
            return false;
        this.vars[index].value = (this.vars[index] instanceof VarTypeReference) ? this.pointerToAddress(value, instanceMapping) : value;
        return true;
    });
}
